import Vue from 'vue';
import {
	getUiOption,
	getNormalAttr,
	getCombining,
	getType,
	isCombining,
	allCombineTypes,
	mergeDefinitions,
	isNest,
	float2percent,
} from '../utils/public';
import BooleanField from './BooleanField.vue';
import NullField from './NullField.vue';
import StringField from './StringField.vue';
import NumberField from './NumberField.vue';
export default Vue.extend({
	components: {
		ObjectField: () => import('./ObjectField.vue'),
		ArrayField: () => import('./ArrayField.vue'),
		NumberField,
		BooleanField,
		NullField,
		StringField,
		IfThenElseField: () => import('./IfThenElse.vue'),
		OneOfField: () => import('./OneOfField.vue'),
		AnyOfField: () => import('./AnyOfField.vue'),
		AllOfField: () => import('./AllOfField.vue'),
	},
	props: {
		schema: {
			type: Object,
			required: true,
			default: () => {},
		},
		uiSchema: {
			type: Object,
			default: () => {},
		},
		definitions: {
			type: Object,
			required: true,
			default: () => {},
		},
		formData: {
			type: [Object, Number, String, Boolean, Array],
			default: undefined,
		},
		formKey: {
			type: String,
			default: undefined,
		},
		errorSchema: {
			type: Object,
			default: () => {},
		},
		errorList: {
			type: Array,
			default: () => [],
		},
		nodePath: {
			type: String,
			default: undefined,
		},
		// 类别属性
		required: {
			type: Boolean,
			default: false,
		},
		nest: {
			type: Boolean,
			default: false,
		},
		funcs: {
			type: Array,
			default: () => [],
		},
		// 仅anyOf/oneOf用于判断获取ui schema
		isArray: {
			type: Boolean,
			default: false,
		},
	},
	data() {
		return {
			// ui schema定义的通用属性
			hidden: false,
			showTitle: true,
			showDescription: true,
			classNames: [] as string[],
			width: '100%',
			titleLocation: 'top',
			// 属性名称
			label: undefined as string | undefined,
			// 描述
			description: undefined as string | undefined,
			// eslint-disable-next-line @typescript-eslint/no-explicit-any
			newSchema: this.schema as any,
			// 组件递归嵌套控制
			isNest: this.nest,
			showNest: this.nest,
			// eslint-disable-next-line @typescript-eslint/no-explicit-any
			midSchema: {} as any,
			// eslint-disable-next-line @typescript-eslint/no-explicit-any
			nowValue: undefined as undefined | any, // 当前字段值
			errorMsg: undefined as undefined | string, // 错误信息
		};
	},
	created() {
		this.getUiGeneralAttr();
		if (this.nest) {
			if (Object.prototype.hasOwnProperty.call(this.schema, '$ref')) {
				if (!this.formData) {
					this.showNest = true;
				} else {
					this.getNewSchema();
					this.newSchema = this.midSchema;
					this.showNest = false;
				}
			} else {
				this.showNest = false;
			}
		} else {
			this.showNest = false;
			if (Object.prototype.hasOwnProperty.call(this.schema, '$ref')) {
				this.getNewSchema();
				this.newSchema = this.midSchema;
			}
		}
		this.label = this.getLabel();
		this.description = this.getDescription();
		this.matchErrorMsg(this.errorList);
		this.width = float2percent(getUiOption(this.uiSchema, 'width'));
	},
	methods: {
		// ui schema定义的通用属性
		getUiGeneralAttr() {
			const attrs = [
				'hidden',
				'showTitle',
				'showDescription',
				'classNames',
				'titleLocation',
			];
			for (let i = 0; i < attrs.length; i++) {
				const attrVal = getUiOption(this.uiSchema, attrs[i]);
				if (attrVal !== undefined) {
					this[attrs[i]] = attrVal;
				}
			}
		},
		// 类型
		getType() {
			return getType(this.newSchema);
		},
		// 组件
		getFieldComponent() {
			const type = this.getType();
			const comb = isCombining(this.newSchema);
			if (comb) {
				if (this.formKey && allCombineTypes.includes(this.formKey)) {
					this.showDescription = false;
					return this.formKey + 'Field';
				}
			}
			if (
				getNormalAttr(this.newSchema, 'if') &&
				this.formKey === 'IfThenElse'
			) {
				return this.formKey + 'Field';
			}
			if (type === 'integer') {
				return 'number-field';
			}
			if (type === undefined) {
				if (comb) {
					const combTypes = getCombining(this.newSchema);
					this.showDescription = false;
					return combTypes[0] + 'Field';
				}
				return 'null-field';
			}
			return type + '-field';
		},
		// 标签
		getLabel() {
			if (this.uiSchema && getUiOption(this.uiSchema, 'title')) {
				return getUiOption(this.uiSchema, 'title');
			} else if (getNormalAttr(this.newSchema, 'title')) {
				return getNormalAttr(this.newSchema, 'title');
			} else {
				return undefined;
			}
		},
		// 描述
		getDescription() {
			if (this.uiSchema) {
				if (getUiOption(this.uiSchema, 'description')) {
					return getUiOption(this.uiSchema, 'description');
				}
			}
			if (getNormalAttr(this.newSchema, 'description')) {
				return getNormalAttr(this.newSchema, 'description');
			}
			return undefined;
		},
		// 引用的子模式
		getNewSchema() {
			const res = mergeDefinitions(this.newSchema, this.definitions);
			if (isNest(res)) {
				this.isNest = true;
				this.midSchema = res;
			} else {
				this.isNest = false;
				this.midSchema = res;
			}
		},
		// 递归ref解析
		addNest() {
			this.getNewSchema();
			this.newSchema = this.midSchema;
			this.showNest = false;
		},
		// 填写表单值
		setValue(val) {
			this.nowValue = val[0];
			if (val.length > 1) {
				this.$emit('setValue', val[0], val[1]);
			} else {
				this.$emit('setValue', val[0]);
			}
		},
		setError(msg) {
			this.errorMsg = msg;
		},
		clearError() {
			this.errorMsg = undefined;
		},
		// 匹配错误信息
		matchErrorMsg(errorList) {
			// const isCombine = isCombining(this.schema);
			if (Array.isArray(errorList) && errorList.length > 0) {
				const path = this.nodePath
					? this.nodePath.replace('/IfThenElse', '')
					: this.nodePath;
				errorList.forEach((item, index) => {
					if (item.property === path) {
						// 联合类型校验错误信息不回显
						// if (
						// 	(isCombine && this.formKey === item.name) ||
						// 	(!isCombine && this.formKey !== undefined)
						// ) {
						// 	this.errorMsg = item.message;
						// 	errorList.splice(index, 1);
						// }
						this.errorMsg = item.message;
						errorList.splice(index, 1);
					}
				});
			} else {
				this.errorMsg = undefined;
			}
		},
	},
});
